@@ -60,7 +60,7 @@ class SrmClient(BaseClient):
         containing its moref, state and list of protection group morefs in this plan.
         """
         data = self._send('ListPlans.xml')
-        morefs = [r['#text'] for r in _listify(data['ListPlansResponse']['returnval'])]
+        morefs = [r['#text'] for r in _listify(data['ListPlansResponse'].get('returnval', []))]
         ret = {}
         for moref in morefs:
             data = self._send('RecoveryPlanGetInfo.xml', moref=moref)['RecoveryPlanGetInfoResponse']['returnval']
@@ -189,7 +189,7 @@ class InternalSrmClient(BaseClient):
             info = _get_proprety(item, 'info').val
             peer = _get_proprety(item, 'peer').val
             device = _get_proprety(item, 'device').val.get('DrStorageStorageDevice', [])
-            replicated_datastore = _get_proprety(item, 'replicatedDatastore').val.get('DrStorageReplicatedDatastore', [])
+            replicated_datastore = _listify(_get_proprety(item, 'replicatedDatastore').val.get('DrStorageReplicatedDatastore', []))
             devices = [dict(name=device.name, role=device.role, id=device.id) for device in _listify(device)]
             for datastore in replicated_datastore:
                 for device in devices:
@@ -197,7 +197,7 @@ class InternalSrmClient(BaseClient):
                         device['key'] = datastore.key['#text']
             pairs.append(dict(key=item.obj['#text'], id=info.key, name=info.name, peer_id=peer.arrayId, devices=devices))
 
-        objects = _listify(munchify(response).RetrievePropertiesExResponse.returnval.objects)
+        objects = _listify(munchify(response).RetrievePropertiesExResponse.get('returnval', dict(objects=[]))['objects'])
 
         for item in objects:
             if item.obj['#text'].startswith('storage-arraymanager'):
@@ -239,8 +239,8 @@ class InternalSrmClient(BaseClient):
                 task_key = value['returnval']['#text']
             else:
                 for key, obj in value['returnval'].items():
-                    if key.endswith('Task'):
-                        task_key = obj['task']['#text']
+                    if key.endswith('Task') and '#text' in obj:
+                        task_key = obj['#text']
 
         specSet = [dict(propSet=[dict(type="Task", all=True)],
                         objectSet=[dict(obj=dict(type="Task", value=task_key), partialUpdates=False, selectSet=[])])]
@@ -267,7 +267,7 @@ class InternalSrmClient(BaseClient):
                                                                                                 selectSet=[dict(name="toPg"),
                                                                                                            dict(type="DrReplicationProtectionGroup", path="protectedVm")])])])])])
 
-        objects = _listify(munchify(response).RetrievePropertiesExResponse.returnval.objects)
+        objects = _listify(munchify(response).RetrievePropertiesExResponse.get('returnval', dict(objects=[]))['objects'])
         protection_groups = []
         protected_vms = {}
 
@@ -309,7 +309,7 @@ class InternalSrmClient(BaseClient):
     def get_unprotected_datastores(self):
         datastores = []
         unassigned_groups = munchify(self._send('QueryUnassignedDatastoreGroupArrays.xml'))
-        for array in _listify(unassigned_groups.QueryUnassignedDatastoreGroupArraysResponse.returnval):
+        for array in _listify(unassigned_groups.QueryUnassignedDatastoreGroupArraysResponse.get('returnval', [])):
             unassigned_datastores = munchify(self._send('QueryUnassignedDatastoreGroups.xml', key=array['#text']))
             for datastore_group in _listify(unassigned_datastores.QueryUnassignedDatastoreGroupsResponse.returnval):
                 datastores.append(dict(key=datastore_group.key, vms=[vm.key['#text'] for vm in  _listify(datastore_group.vm)]))
@@ -337,9 +337,8 @@ class InternalSrmClient(BaseClient):
                                                 objectSet=[dict(obj=dict(type="DrReplicationReplicationManager", value="DrReplicationManager"), partialUpdates=False,
                                                                 selectSet=[dict(type="DrReplicationReplicationManager", path="replicationProvider", skip=True,
                                                                                 selectSet=[dict(type="DrReplicationStorageProvider", path="storageManager", skip=True,
-                                                                                                selectSet=[dict(type="DrStorageStorageManager", path="arrayManager",
-                                                                                                                selectSet=[dict(type="DrStorageArrayManager", path="adapter")])])])])])])
-        objects = _listify(response['RetrievePropertiesExResponse']['returnval']['objects'])
+                                                                                                selectSet=[dict(type="DrStorageStorageManager", path="adapter")])])])])])
+        objects = _listify(response['RetrievePropertiesExResponse'].get('returnval', dict(objects=[]))['objects'])
         adapters = []
         for item in objects:
             model = _listify(_get_proprety(item, 'arrayModel')['val'].get('DrStorageStorageAdapterArrayModel', []))[0]
@@ -353,9 +352,9 @@ class InternalSrmClient(BaseClient):
                                  connection_parameters=parameters))
         return adapters
 
-    def create_adapter(self, adapter, name, connection_parameters):
+    def create_array(self, adapter, name, connection_parameters):
         self.wait_for_task(self._send('StorageManagerCreateArrayManager2_Task.xml',
                                       key=adapter['key'], name=name, connection_parameters=connection_parameters))
 
-    def delete_adapter(self, adapter):
+    def delete_array(self, adapter):
         self.wait_for_task(self._send('StorageManagerDeleteArrayManager2_Task.xml', key=adapter['key']))
